import java.io.*;
import java.util.*;

/****************************************/
/* Clases que representan la tabla      */
interface RegistroRAF {
	public void leer(RandomAccessFile f) throws IOException;
	public void guardar(RandomAccessFile f) throws IOException;
	public long largo();
}

class TablaRAF {
	private RandomAccessFile raf;

	public TablaRAF(String tabla) {
		try {
			raf = new RandomAccessFile(tabla, "rw");
		}
		catch (IOException e) {
			TablaRAF.reportarError("No se puede crear la tabla", e);
		}
	}

	public void agregarRegistro(RegistroRAF r) {
		try {
			raf.seek(raf.length());
			r.guardar(raf);
		}
		catch (IOException e) {
			TablaRAF.reportarError("No se puede guardar el registro", e);
		}
	}

	public void cambiarRegistro(RegistroRAF r, int n) {
		try {
			raf.seek(r.largo() * n);
			r.guardar(raf);
		}
		catch (IOException e) {
			TablaRAF.reportarError("No se puede guardar el registro", e);
		}
	}

	public void leerRegistro(RegistroRAF r, int n) {
		try {
			raf.seek(r.largo() * n);
			r.leer(raf);
		}
		catch (IOException e) {
			TablaRAF.reportarError("No se puede leer el registro", e);
			r = null;
		}
	}

	public void borrarRegistro(RegistroRAF r) {
			TablaRAF.reportarError("Este mtodo no est implementado", null);
	}

	public int nRegistros(RegistroRAF r) {
		try {
			return (int) (raf.length() / r.largo());
		}
		catch (IOException e) {
			TablaRAF.reportarError("No se puede abrir la tabla", e);
			return -1;
		}
	}

	public void cerrar() {
		try {
			raf.close();
		}
		catch (IOException e) {
			TablaRAF.reportarError("No se puede cerrar la tabla", e);
		}
	}

	static public String toUTF(String t, int n) {
		String s = t;
		if (s.length() > n)
			return s.substring(0, n);
		for (int i=s.length(); i<n; i++)
			s += " ";
		return s;
	}

	static public void reportarError(String txt, Exception e) {
		System.err.println("********************************************* " + new Date(System.currentTimeMillis()));
		System.err.println("ERROR: " + txt);
		if (e != null) {
			e.printStackTrace();
		}
	}
}

/*************************************/
/* Definicin de la Base de Datos    */
class Alumno implements RegistroRAF {
	public int matricula;		// 4 bytes
	public String nombre;		// 100 bytes + 2 bytes control
	public String email;		// 50 bytes + 2 bytes control
	public final int largo = 4 + 102 + 52;

	public Alumno() {
		this.matricula = 0;
		this.nombre = "";
		this.email = "";
	}

	public Alumno(int m, String n, String e) {
		this.matricula = m;
		this.nombre = n;
		this.email = e;
	}

	public void leer(RandomAccessFile f) throws IOException {
		this.matricula = f.readInt();
		this.nombre = f.readUTF().trim();
		this.email = f.readUTF().trim();
	}

	public void guardar(RandomAccessFile f) throws IOException {
		f.writeInt(this.matricula);
		f.writeUTF(TablaRAF.toUTF(this.nombre, 100));
		f.writeUTF(TablaRAF.toUTF(this.email, 50));
	}

	public long largo() {
		return this.largo;
	}
}

class Curso implements RegistroRAF {
	public String codigo;		// 5 bytes + 2 bytes de control
	public int seccion;			// 4 bytes
	public String nombre;		// 50 bytes + 2 bytes de control
	public String sala;			// 10 bytes + 2 bytes de control
	public final int largo = 7 + 4 + 52 + 12;

	public Curso() {
		this.codigo = "";
		this.seccion = 0;
		this.nombre = "";
		this.sala = "";
	}

	public Curso(String c, int s, String n, String sl) {
		this.codigo = c;
		this.seccion = s;
		this.nombre = n;
		this.sala = sl;
	}

	public void leer(RandomAccessFile f) throws IOException {
		this.codigo = f.readUTF().trim();
		this.seccion = f.readInt();
		this.nombre = f.readUTF().trim();
		this.sala = f.readUTF().trim();
	}

	public void guardar(RandomAccessFile f) throws IOException {
		f.writeUTF(TablaRAF.toUTF(this.codigo, 5));
		f.writeInt(this.seccion);
		f.writeUTF(TablaRAF.toUTF(this.nombre, 50));
		f.writeUTF(TablaRAF.toUTF(this.sala, 10));
	}

	public long largo() {
		return this.largo;
	}
}

class Inscripcion implements RegistroRAF {
	public int matricula_alumno;	// 4 bytes
	public String codigo_curso;		// 5 bytes + 2 bytes de control
	public int seccion_curso;		// 4 bytes
	public final int largo = 4 + 7 + 4;

	public Inscripcion() {
		this.matricula_alumno = 0;
		this.codigo_curso = "";
		this.seccion_curso = 0;
	}

	public Inscripcion(int ma, String cc, int sc) {
		this.matricula_alumno = ma;
		this.codigo_curso = cc;
		this.seccion_curso = sc;
	}

	public void leer(RandomAccessFile f) throws IOException {
		this.matricula_alumno = f.readInt();
		this.codigo_curso = f.readUTF().trim();
		this.seccion_curso = f.readInt();
	}

	public void guardar(RandomAccessFile f) throws IOException {
		f.writeInt(this.matricula_alumno);
		f.writeUTF(TablaRAF.toUTF(this.codigo_curso, 5));
		f.writeInt(this.seccion_curso);
	}

	public long largo() {
		return this.largo;
	}
}

/****************************************/
/* Programa de uso de la Base de Datos  */
public class RAF {
	static public void agregarDatos(Console c) {
		TablaRAF table;
		c.print("Tabla: (A)lumnos - (C)ursos - (I)nscripcin ?");
		char tab = c.readLine().charAt(0);

		if (tab == 'A' || tab == 'a') {
			table = new TablaRAF("alumnos");
			Alumno regis = new Alumno();
			c.print("[Alumno] N de Matrcula >");
			regis.matricula = c.readInt();
			c.print("[Alumno] Nombre          >");
			regis.nombre = c.readLine();
			c.print("[Alumno] e-Mail          >");
			regis.email = c.readLine();
			table.agregarRegistro(regis);
			table.cerrar();
			c.println("El Alumno ha sido agregado");
		}
		else if (tab == 'C' || tab == 'c') {
			table = new TablaRAF("cursos");
			Curso regis = new Curso();
			c.print("[Curso] Codigo          >");
			regis.codigo = c.readLine();
			c.print("[Curso] Seccin         >");
			regis.seccion = c.readInt();
			c.print("[Curso] Nombre          >");
			regis.nombre = c.readLine();
			c.print("[Curso] Sala            >");
			regis.sala = c.readLine();
			table.agregarRegistro(regis);
			table.cerrar();
			c.println("El Curso ha sido agregado");
		}
		else if (tab == 'I' || tab == 'i') {
			boolean existe;

			table = new TablaRAF("inscripciones");
			Inscripcion regis = new Inscripcion();
			c.print("[Alumno] N de Matrcula >");
			regis.matricula_alumno = c.readInt();

			TablaRAF table_alumnos = new TablaRAF("alumnos");
			Alumno r1 = new Alumno();
			existe = false;
			for (int i=0; i<table_alumnos.nRegistros(r1); i++) {
				table_alumnos.leerRegistro(r1, i);
				existe = existe || (r1.matricula == regis.matricula_alumno);
			}
			table_alumnos.cerrar();
			if (!existe) {
				c.println(regis.matricula_alumno + " no es un alumno vlido");
				return;
			}

			c.print("[Curso] Codigo           >");
			regis.codigo_curso = c.readLine();
			c.print("[Curso] Seccin          >");
			regis.seccion_curso = c.readInt();

			TablaRAF table_cursos = new TablaRAF("cursos");
			Curso r2 = new Curso();
			existe = false;
			for (int i=0; i<table_cursos.nRegistros(r2); i++) {
				table_cursos.leerRegistro(r2, i);
				existe = existe || (r2.codigo.equals(regis.codigo_curso) && r2.seccion == regis.seccion_curso);
			}
			table_alumnos.cerrar();
			if (!existe) {
				c.println(regis.codigo_curso + regis.seccion_curso + " no es un curso vlido");
				return;
			}
			table.agregarRegistro(regis);
			c.println("La Inscripcin ha sido agregada");
		}
		else
			c.println(tab + " no es una tabla vlida");
	}

	static public void consultarDatos(Console c) {
		TablaRAF table;
		c.print("Consulta: (A)lumnos de un curso - (C)ursos de un alumno - Cursos en una (S)ala ?");
		char tab = c.readLine().charAt(0);

		if (tab == 'A' || tab == 'a') {
			c.print("Cdigo del curso ?");
			String codigo = c.readLine();

			TablaRAF inscritos = new TablaRAF("inscripciones");
			Inscripcion inscripcion = new Inscripcion();
			int n = 0;
			for (int i=0; i<inscritos.nRegistros(inscripcion); i++) {
				inscritos.leerRegistro(inscripcion, i);
				if (inscripcion.codigo_curso.equals(codigo)) {
					TablaRAF alumnos = new TablaRAF("alumnos");
					Alumno alumno = new Alumno();
					for (int j=0; j<alumnos.nRegistros(alumno); j++) {
						alumnos.leerRegistro(alumno, j);
						if (alumno.matricula == inscripcion.matricula_alumno) {
							c.println(alumno.matricula + " - " + alumno.nombre + " (" + alumno.email + ")");
							n++;
						}
					}
				}
			}
			c.println(n + " registros obtenidos");
		}
		else if (tab == 'C' || tab == 'C') {
			c.print("Matrcula del alumno ?");
			int matricula = c.readInt();

			TablaRAF inscritos = new TablaRAF("inscripciones");
			Inscripcion inscripcion = new Inscripcion();
			int n = 0;
			for (int i=0; i<inscritos.nRegistros(inscripcion); i++) {
				inscritos.leerRegistro(inscripcion, i);
				if (inscripcion.matricula_alumno == matricula) {
					TablaRAF cursos = new TablaRAF("cursos");
					Curso curso = new Curso();
					for (int j=0; j<cursos.nRegistros(curso); j++) {
						cursos.leerRegistro(curso, j);
						if (curso.codigo.equals(inscripcion.codigo_curso) && curso.seccion == inscripcion.seccion_curso) {
							c.println(curso.codigo + "-" + curso.seccion + " " + curso.nombre + " (" + curso.sala + ")");
							n++;
						}
					}
				}
			}
			c.println(n + " registros obtenidos");
		}
		else if (tab == 'S' || tab == 's') {
			c.print("Sala ?");
			String sala = c.readLine();

			TablaRAF cursos = new TablaRAF("cursos");
			Curso curso = new Curso();
			int n = 0;
			for (int i=0; i<cursos.nRegistros(curso); i++) {
				cursos.leerRegistro(curso, i);
				if (curso.sala.equals(sala)) {
					c.println(curso.codigo + "-" + curso.seccion + " " + curso.nombre);
					n++;
				}
			}
			c.println(n + " registros obtenidos");
		}
		else
			c.println(tab + " no es una consulta vlida");
	}

	static public void main(String[] args) {
		Console c = new Console();
		while(true) {
			c.print("Accin: (A)gregar Datos - (C)onsultar Datos - (S)alir ?");
			char acc = c.readLine().charAt(0);

			if (acc == 'A' || acc == 'a')
				agregarDatos(c);
			else if (acc == 'C' || acc == 'c')
				consultarDatos(c);
			else if (acc == 'S' || acc == 's')
				break;
			else
				c.println(acc + " no es una accin vlida");
		}
		System.exit(0);
	}
}